@@ -18,7 +18,9 @@
 #include <glib/gstdio.h>  // for g_unlink
 
 #include <js/Array.h>
+#include <js/BigInt.h>
 #include <js/CharacterEncoding.h>
+#include <js/Id.h>
 #include <js/RootingAPI.h>
 #include <js/TypeDecls.h>
 #include <js/Utility.h>  // for UniqueChars
@@ -26,6 +28,7 @@
 #include <js/ValueArray.h>
 #include <jsapi.h>
 #include <jspubtd.h>  // for JSProto_Number
+#include <mozilla/Span.h>  // for MakeStringSpan
 
 #include "gi/arg-inl.h"
 #include "gjs/context.h"
@@ -44,8 +47,12 @@ void g_assertion_message(const char*, const char*, int, const char*,
 
 #define VALID_UTF8_STRING "\303\211\303\226 foobar \343\203\237"
 
+namespace Gjs {
+namespace Test {
 static unsigned cpp_random_seed = 0;
 
+using Gjs::Test::assert_equal;
+
 template <typename T>
 T get_random_number() {
     std::mt19937_64 gen(cpp_random_seed);
@@ -67,23 +74,6 @@ T get_random_number() {
     }
 }
 
-template <typename T>
-constexpr void assert_equal(T a, T b) {
-    if constexpr (std::is_integral_v<T>) {
-        if constexpr (std::is_unsigned_v<T>)
-            g_assert_cmpuint(a, ==, b);
-        else
-            g_assert_cmpint(a, ==, b);
-    } else if constexpr (std::is_arithmetic_v<T>) {
-        g_assert_cmpfloat_with_epsilon(a, b, std::numeric_limits<T>::epsilon());
-    } else if constexpr (std::is_same_v<T, char*>) {
-        g_assert_cmpstr(a, ==, b);
-    } else if constexpr (std::is_pointer_v<T>) {
-        assert_equal(reinterpret_cast<uintptr_t>(a),
-                     reinterpret_cast<uintptr_t>(b));
-    }
-}
-
 static void
 gjstest_test_func_gjs_context_construct_destroy(void)
 {
@@ -574,6 +564,63 @@ static void test_jsapi_util_debug_string_object_with_complicated_to_string(
     g_assert_cmpstr(u8"🍪,🍩", ==, debug_output.c_str());
 }
 
+static void test_gjs_debug_id_string_no_quotes(GjsUnitTestFixture* fx,
+                                               const void*) {
+    jsid id = gjs_intern_string_to_id(fx->cx, "prop_key");
+    std::string debug_output = gjs_debug_id(id);
+
+    g_assert_cmpstr(debug_output.c_str(), ==, "prop_key");
+}
+
+static void test_gjs_debug_string_quotes(GjsUnitTestFixture* fx, const void*) {
+    JS::ConstUTF8CharsZ chars("a string", strlen("a string"));
+    JSString* str = JS_NewStringCopyUTF8Z(fx->cx, chars);
+    std::string debug_output = gjs_debug_string(str);
+
+    g_assert_cmpstr(debug_output.c_str(), ==, "\"a string\"");
+}
+
+static void test_gjs_debug_value_bigint(GjsUnitTestFixture* fx, const void*) {
+    JS::BigInt* bi = JS::NumberToBigInt(fx->cx, 42);
+    JS::Value v = JS::BigIntValue(bi);
+    std::string debug_output = gjs_debug_value(v);
+
+    g_assert_cmpstr(debug_output.c_str(), ==, "42n");
+}
+
+static void test_gjs_debug_value_bigint_uint64(GjsUnitTestFixture* fx,
+                                               const void*) {
+    // gjs_debug_value(BigIntValue) prints whatever fits into int64_t, because
+    // more complicated operations might be fallible
+    JS::BigInt* bi = JS::NumberToBigInt(fx->cx, G_MAXUINT64);
+    JS::Value v = JS::BigIntValue(bi);
+    std::string debug_output = gjs_debug_value(v);
+
+    g_assert_cmpstr(debug_output.c_str(), ==, "-1n");
+}
+
+static void test_gjs_debug_value_bigint_huge(GjsUnitTestFixture* fx,
+                                             const void*) {
+    JS::BigInt* bi = JS::SimpleStringToBigInt(
+        fx->cx, mozilla::MakeStringSpan("10000000000000001"), 16);
+    JS::Value v = JS::BigIntValue(bi);
+    std::string debug_output = gjs_debug_value(v);
+
+    g_assert_cmpstr(debug_output.c_str(), ==, "1n");
+}
+
+static void test_gjs_debug_value_string_quotes(GjsUnitTestFixture* fx,
+                                               const void*) {
+    JS::RootedValue v(fx->cx);
+    bool ok = gjs_string_from_utf8(fx->cx, "a string", &v);
+
+    g_assert_true(ok);
+
+    std::string debug_output = gjs_debug_value(v);
+
+    g_assert_cmpstr(debug_output.c_str(), ==, "\"a string\"");
+}
+
 static void
 gjstest_test_func_util_misc_strv_concat_null(void)
 {
@@ -833,10 +880,15 @@ static void gjstest_test_func_gjs_context_argv_array() {
     g_assert_false(ok);
 }
 
+}  // namespace Test
+}  // namespace Gjs
+
 int
 main(int    argc,
      char **argv)
 {
+    using namespace Gjs::Test;  // NOLINT(build/namespaces)
+
     /* Avoid interference in the tests from stray environment variable */
     g_unsetenv("GJS_ENABLE_PROFILER");
     g_unsetenv("GJS_TRACE_FD");
@@ -931,12 +983,21 @@ main(int    argc,
     ADD_JSAPI_UTIL_TEST("gi/args/safe-integer/min",
                         gjstest_test_safe_integer_min);
 
+    // Debug functions
+    ADD_JSAPI_UTIL_TEST("debug_id/string/no-quotes",
+                        test_gjs_debug_id_string_no_quotes);
+    ADD_JSAPI_UTIL_TEST("debug_string/quotes", test_gjs_debug_string_quotes);
+    ADD_JSAPI_UTIL_TEST("debug_value/bigint", test_gjs_debug_value_bigint);
+    ADD_JSAPI_UTIL_TEST("debug_value/bigint/uint64",
+                        test_gjs_debug_value_bigint_uint64);
+    ADD_JSAPI_UTIL_TEST("debug_value/bigint/huge",
+                        test_gjs_debug_value_bigint_huge);
+    ADD_JSAPI_UTIL_TEST("debug_value/string/quotes",
+                        test_gjs_debug_value_string_quotes);
+
 #undef ADD_JSAPI_UTIL_TEST
 
     gjs_test_add_tests_for_coverage ();
-    gjs_test_add_tests_for_parse_call_args();
-    gjs_test_add_tests_for_rooting();
-    gjs_test_add_tests_for_jsapi_utils();
 
     g_test_run();
 
